# ArtifactTypeModelManifest specifies the artifact type for a model manifest.
ARTIFACTTYPEMODELMANIFEST = "application/vnd.cncf.model.manifest.v1+json"

# MediaTypeModelConfig specifies the media type for a model configuration.
MEDIATYPEMODELCONFIG = "application/vnd.cncf.model.config.v1+json"

# MediaTypeModelWeightRaw is the media type used for an unarchived, uncompressed model weights.
MEDIATYPEMODELWEIGHTRAW = "application/vnd.cncf.model.weight.v1.raw"

# MediaTypeModelWeight is the media type used for model weights.
MEDIATYPEMODELWEIGHT = "application/vnd.cncf.model.weight.v1.tar"

# MediaTypeModelWeightGzip is the media type used for gzipped model weights.
MEDIATYPEMODELWEIGHTGZIP = "application/vnd.cncf.model.weight.v1.tar+gzip"

# MediaTypeModelWeightZstd is the media type used for zstd compressed model weights.
MEDIATYPEMODELWEIGHTZSTD = "application/vnd.cncf.model.weight.v1.tar+zstd"

# MediaTypeModelWeightConfigRaw is the media type used for an unarchived, uncompressed model weights, including files like `tokenizer.json`, `config.json`, etc.
MEDIATYPEMODELWEIGHTCONFIGRAW = "application/vnd.cncf.model.weight.config.v1.raw"

# MediaTypeModelConfig specifies the media type for configuration of the model weights, including files like `tokenizer.json`, `config.json`, etc.
MEDIATYPEMODELWEIGHTCONFIG = "application/vnd.cncf.model.weight.config.v1.tar"

# MediaTypeModelConfigGzip specifies the media type for gzipped configuration of the model weights, including files like `tokenizer.json`, `config.json`, etc.
MEDIATYPEMODELWEIGHTCONFIGGZIP = "application/vnd.cncf.model.weight.config.v1.tar+gzip"

# MediaTypeModelConfigZstd specifies the media type for zstd compressed configuration of the model weights, including files like `tokenizer.json`, `config.json`, etc.
MEDIATYPEMODELWEIGHTCONFIGZSTD = "application/vnd.cncf.model.weight.config.v1.tar+zstd"

# MediaTypeModelDocRaw is the media type used for an unarchived, uncompressed model documentation, including documentation files like `README.md`, `LICENSE`, etc.
MEDIATYPEMODELDOCRAW = "application/vnd.cncf.model.doc.v1.raw"

# MediaTypeModelDoc specifies the media type for model documentation, including documentation files like `README.md`, `LICENSE`, etc.
MEDIATYPEMODELDOC = "application/vnd.cncf.model.doc.v1.tar"

# MediaTypeModelDocGzip specifies the media type for gzipped model documentation, including documentation files like `README.md`, `LICENSE`, etc.
MEDIATYPEMODELDOCGZIP = "application/vnd.cncf.model.doc.v1.tar+gzip"

# MediaTypeModelDocZstd specifies the media type for zstd compressed model documentation, including documentation files like `README.md`, `LICENSE`, etc.
MEDIATYPEMODELDOCZSTD = "application/vnd.cncf.model.doc.v1.tar+zstd"

# MediaTypeModelCodeRaw is the media type used for an unarchived, uncompressed model code, including code artifacts like scripts, code files etc.
MEDIATYPEMODELCODERAW = "application/vnd.cncf.model.code.v1.raw"

# MediaTypeModelCode specifies the media type for model code, including code artifacts like scripts, code files etc.
MEDIATYPEMODELCODE = "application/vnd.cncf.model.code.v1.tar"

# MediaTypeModelCodeGzip specifies the media type for gzipped model code, including code artifacts like scripts, code files etc.
MEDIATYPEMODELCODEGZIP = "application/vnd.cncf.model.code.v1.tar+gzip"

# MediaTypeModelCodeZstd specifies the media type for zstd compressed model code, including code artifacts like scripts, code files etc.
MEDIATYPEMODELCODEZSTD = "application/vnd.cncf.model.code.v1.tar+zstd"

# MediaTypeModelDatasetRaw is the media type used for an unarchived, uncompressed model datasets, including datasets that may be needed throughout the lifecycle of AI/ML models.
MEDIATYPEMODELDATASETRAW = "application/vnd.cncf.model.dataset.v1.raw"

# MediaTypeModelDataset specifies the media type for model datasets, including datasets that may be needed throughout the lifecycle of AI/ML models.
MEDIATYPEMODELDATASET = "application/vnd.cncf.model.dataset.v1.tar"

# MediaTypeModelDatasetGzip specifies the media type for gzipped model datasets, including datasets that may be needed throughout the lifecycle of AI/ML models.
MEDIATYPEMODELDATASETGZIP = "application/vnd.cncf.model.dataset.v1.tar+gzip"

# MediaTypeModelDatasetZstd specifies the media type for zstd compressed model datasets, including datasets that may be needed throughout the lifecycle of AI/ML models.
MEDIATYPEMODELDATASETZSTD = "application/vnd.cncf.model.dataset.v1.tar+zstd"
